
// Example: performance impact of memory references locality
// Compile: gcc -std=c99 -O2

#include <stdio.h>
#include <time.h>       // clock()

#define N 10000         // value depends on available memory
int p[N][N];

int main(int argc, char *argv[])
{
    // fill matrix (and CPU cache)
    for (int i = 0; i < N; ++i)
        for (int j = 0; j < N; ++j)
            p[i][j] = i+j;
    printf("%dx%d int matrix sum:\n", N, N);
    long long sum1=0;
    {   // row-major access
        clock_t start = clock();
        for (int i = 0; i < N; ++i)
            for (int j = 0; j < N; ++j)
                sum1+=p[i][j];
        clock_t end = clock();
        printf("%.3f s\t",(end-start)/(double)CLOCKS_PER_SEC);
        printf("using row-major access\n");
    }
    long long sum2=0;
    {   // column-major access (bad locality in C, good in FORTRAN)
        clock_t start = clock();
        for (int j = 0; j < N; ++j)
            for (int i = 0; i < N; ++i)
                sum2+=p[i][j];
        clock_t end = clock();
        printf("%.3f s\t",(end-start)/(double)CLOCKS_PER_SEC);
        printf("using column-major access\n");
    }
    return sum1!=sum2;
}

