
// Use  gcc -O1 -march=native -funroll-loops

/*  This example is inspired by:
 *  "Reciprocal Multiplication, a tutorial"
 *  Part of the Arithmetic Tutorial Collection by Douglas W. Jones
 *  THE UNIVERSITY OF IOWA Department of Computer Science
 *  Copyright © 1999, Douglas. W. Jones, with major revisions made in 2002.
 */

#include <stdio.h>      // printf
#include <stdint.h>     // uint64_t, uint32_t
#include <time.h>       // clock()

// divide x by 10
inline uint32_t div10(uint32_t x) {
    uint64_t a = x;
    uint32_t point8 = 0xCCCCCCCD; // rounded 0.8 in fixed-point representation
    return (a*point8)>>(32+3);    // integer part of fixed point result / 8
}

#define CHECK(op) do { \
    clock_t start = clock(); \
    uint32_t r1=0; \
    uint32_t i=0; do { r1 += op; ++i; } while(i!=0);\
    clock_t stop = clock(); \
    printf("%-8s: %g s (sum=%lu)\n", #op, (stop-start)*1.0/CLOCKS_PER_SEC, r1); \
    }while(0)

// test all possible values
int main() {
    printf("Benchmark of division using modular inverse multiplication:\n");
    CHECK(div10(i));
    CHECK(i/10);
    CHECK(div10(i));
    CHECK(i/10);
}

