/* test-copy.c 
 * 
 * test rychlosti koprovn souboru
 *   - znak po znaku
 *   - po blocch
 *
 * pouijte dlouh soubor (men nen pli pesn)
 * pozorujte vliv systmovch vyrovnvacch pamt
 *
 * Pozor -- pepisuje clov soubor bez dotazu
 * 
 * Pklad (Linux 2.2.18/PIII-733/128MB/GCC -O6):
 *     test-copy testfile10000KB testfile2
 *   vstup:
 *     as koprovn souboru:
 *     copy1: 1.6 s
 *     copy2: 0.08 s       <<< zde je vidt zrychlen
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

typedef void (*copy_function_pointer_t)(FILE*f1,FILE*f2);

/* funkce mec as kopie */
double copy(const char * from, 
            const char * to, 
            copy_function_pointer_t function) {
  clock_t clock_time;
  FILE *f1 = fopen(from,"r");
  FILE *f2 = fopen(to,"w");
  if(f1==NULL || f2==NULL) {
    fprintf(stderr, "Soubor nelze otevrit\n");
    exit(2);
  }
  clock_time = clock();
  function(f1,f2); /* voln testovan funkce */
  clock_time = clock() - clock_time;
  fclose(f1);
  fclose(f2);
  return (double) clock_time / CLOCKS_PER_SEC;
}

/**************************************************************************/

void copy1(FILE*f1,FILE*f2) { /* kopie znak po znaku - neefektivn */
    int c;
    while( (c=getc(f1)) != EOF )
      putc(c,f2); 
}

#define BUFSIZE 16384         /* mete zvtit */
void copy2(FILE*f1,FILE*f2) { /* kopie po blocch znak - efektivn */
  static char buffer[BUFSIZE];
  int n;
  while( (n=fread(buffer,1,BUFSIZE,f1)) != 0 )
    fwrite(buffer,1,n,f2); 
}

/**************************************************************************/

int main(int argc, char *argv[]) { 
  if(argc != 3) {
    fprintf(stderr, "Pouit: %s soubor_odkud soubor_kam\n", argv[0]);
    return 1;
  }
  copy(argv[1], argv[2], copy2); /* napln sys. vyrovnavc pamti */
  printf("as koprovn souboru:\n");  
  printf("copy1: %g s\n", copy(argv[1], argv[2], copy1) );  
  printf("copy2: %g s\n", copy(argv[1], argv[2], copy2) );  
  printf("copy1: %g s\n", copy(argv[1], argv[2], copy1) );  
  printf("copy2: %g s\n", copy(argv[1], argv[2], copy2) );  
  printf("copy1: %g s\n", copy(argv[1], argv[2], copy1) );  
  printf("copy2: %g s\n", copy(argv[1], argv[2], copy2) );  
  return 0;
}

