
// numerical integration (quadrature) example
// trpezoidal and simpson rule comparison

#include <stdio.h>
#include <math.h>
#include <assert.h>

double trapezoidal(double x1, double x2, double(*f)(double), unsigned n) {
    double h = (x2-x1)/n;
    double sum = f(x1)/2 + f(x2)/2;     // end-points
    for(int i=1; i<n; ++i) {
        sum += f(x1+i*h);
    }
    return sum*h;
}

double simpson(double x1, double x2, double(*f)(double), unsigned n) {
    assert((n%2)==0);
    double h = (x2-x1)/n;
    double sum = f(x1)/3 + f(x2)/3;     // end-points
    for(int i=1; i<n; i+=2) {
        sum += 4.0/3.0 * f(x1+i*h);
        if(i<n)
            sum += 2.0/3.0 * f(x1+i*h+h);
    }
    return sum*h;
}

int main() {
    //double pi = M_PI;
    const double pi = 4*atan(1);
    printf("\n%s\t %s\n","trapez(0,pi,sin,50)-2", "trapez(0,2*pi,sin,50)");
    printf("% .16f\t",trapezoidal(0,pi,sin,50)-2);
    printf("% .16f\n",trapezoidal(0,2*pi,sin,50));
    printf("\n%s\t %s\n","simpson(0,pi,sin,50)-2", "simpson(0,2*pi,sin,50)");
    printf("% .16f\t",simpson(0,pi,sin,50)-2);
    printf("% .16f\n",simpson(0,2*pi,sin,50));
    printf("\n");
}

